// Copyright 2016-2021 JetBrains s.r.o.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package com.goide.psi.impl.codeFragment;

import com.goide.psi.GoReferenceExpressionBase;
import com.intellij.psi.PsiElement;
import com.intellij.psi.util.PsiTreeUtil;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

public abstract class GoCodeFragmentResolveTestBase extends GoCodeFragmentTestBase {
  protected void doTestTextEquals(@NotNull Class<?> resolvedElementClass, @NotNull String resolvedElementText) {
    PsiElement resolve = openTestFragmentAndResolve();
    assertNotNull(resolve);
    assertInstanceOf(resolve, resolvedElementClass);
    assertEquals(resolvedElementText, resolve.getText());
  }

  protected void doTestTextStartsWith(@NotNull Class<?> resolvedElementClass, @NotNull String resolvedElementTextPrefix) {
    PsiElement resolve = openTestFragmentAndResolve();
    assertNotNull(resolve);
    assertInstanceOf(resolve, resolvedElementClass);
    assertTrue("Unexpected element text: " + resolve.getText(), resolve.getText().startsWith(resolvedElementTextPrefix));
  }

  @Nullable
  private PsiElement openTestFragmentAndResolve() {
    openFragmentInEditor();
    PsiElement element = findElementAtCaretOrInSelection();
    GoReferenceExpressionBase ref = PsiTreeUtil.getNonStrictParentOfType(element, GoReferenceExpressionBase.class);
    assertNotNull(ref);
    return ref.resolve();
  }
}
