// Copyright 2016-2021 JetBrains s.r.o.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// https://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
package com.goide.tree;

import com.goide.GoCodeInsightFixtureTestCase;
import com.intellij.openapi.util.Disposer;
import com.intellij.openapi.util.io.FileUtil;
import com.intellij.projectView.TestProjectTreeStructure;
import com.intellij.testFramework.PlatformTestUtil;
import com.intellij.ui.tree.TreeVisitor;
import com.intellij.util.ui.tree.TreeUtil;
import org.jetbrains.annotations.NotNull;

import javax.swing.*;
import java.io.File;
import java.io.IOException;

public abstract class GoTreeStructureProviderTestBase extends GoCodeInsightFixtureTestCase {
  protected TestProjectTreeStructure myStructure;

  @Override
  protected void setUp() throws Exception {
    super.setUp();
    myStructure = new TestProjectTreeStructure(getProject(), getTestRootDisposable());
  }

  @Override
  protected void tearDown() throws Exception {
    try {
      Disposer.dispose(myStructure);
      myStructure = null;
    }
    catch (Throwable e) {
      addSuppressedException(e);
    }
    finally {
      super.tearDown();
    }
  }

  @NotNull
  protected String getExpectedTree(@NotNull String expectedFilePath) throws IOException {
    return FileUtil.loadFile(new File(getTestDataPath() + "/" + expectedFilePath), true);
  }

  protected static void expand(@NotNull JTree projectViewTree, @NotNull TreeVisitor visitor) {
    PlatformTestUtil.waitForPromise(TreeUtil.promiseExpand(projectViewTree, visitor));
  }
}
